@@ -23,7 +23,7 @@ use Class::XSAccessor
     };
 use POE;
 
-our $VERSION = '0.3.1';
+our $VERSION = '0.4.0';
 
 sub spawn {
     my ($class, $opts) = @_;
@@ -40,6 +40,7 @@ sub spawn {
     my $session = POE::Session->create(
         inline_states => {
             # public events
+            cpan2dist_status     => \&cpan2dist_status,
             upstream_status      => \&upstream_status,
             local_status         => \&local_status,
             module_spawned       => \&module_spawned,
@@ -82,6 +83,14 @@ sub spawn {
 
 # -- public events
 
+sub cpan2dist_status {
+    my ($k, $h, $module, $status) = @_[KERNEL, HEAP, ARG0, ARG1];
+    # FIXME: what if $status is false
+
+    $k->post($module, 'install_from_local');
+}
+
+
 sub local_status {
     my ($k, $h, $module, $is_installed) = @_[KERNEL, HEAP, ARG0, ARG1];
 
@@ -236,6 +245,12 @@ A list of modules to start packaging.
 The following events are the module's API.
 
 
+=head2 cpan2dist_status( $module, $success )
+
+Sent when C<$module> has been C<cpan2dist>-ed, with C<$success> being true
+if everything went fine.
+
+
 =head2 local_status( $module, $is_installed )
 
 Sent when C<$module> knows whether it is installed locally (C<$is_installed>
